@@ -7,7 +7,7 @@ use Carp;
 use File::Spec;
 use Test::Builder::Module;
 
-our $VERSION = '0.01';
+our $VERSION = '0.02';
 our @ISA = 'Test::Builder::Module';
 
 ##############################
@@ -19,8 +19,12 @@ sub new {
     my $dir = shift;
     my %opts = @_;
 
+
+    $opts{unique} = 1 unless defined $opts{unique};
+    $dir = File::Spec->join(split '/', $dir);
+
     if ($opts{unique}) {
-	mkdir $dir or croak "$dir: $!";
+	mkdir $dir or croak "Failed to create '$dir': $!";
     } else {
 	mkdir $dir;
 	croak "$dir: $!" unless -d $dir;
@@ -39,27 +43,22 @@ sub DESTROY {
 ##############################
 
 sub name {
-    my ($self,$file) = @_;
-    return defined($self->{template})?
-	sprintf($self->{template}, $file):
-	$file;
+    my ($self,$path) = @_;
+    my @path = split /\//, $path;
+    my $file = pop @path;
+    if (ref($self) and defined($self->{template})) {
+      $file = sprintf($self->{template}, $file);
+    };
+    return @path ? File::Spec->catfile(@path,$file) : $file;
 };
 
 sub path {
     my ($self,$file) = @_;
-    File::Spec->catfile($self->{dir}, $self->name($file));
+    return defined($file)?
+      File::Spec->catfile($self->{dir}, $self->name($file)):
+      $self->{dir};
 };
 
-sub check_file {
-    my ($self,$file) = @_;
-    my $rv;
-    if (-f $self->path($file)) {
-      $rv = $self->{files}{$file} = 1;
-    } else {
-      $rv = $self->{files}{$file} = 0;
-    }
-    return $rv;
-}
 
 sub touch {
     my $self = shift;
@@ -85,24 +84,68 @@ sub create {
   return $path;
 }
 
+sub mkdir {
+  my ($self, $dir) = @_;
+  my $path = $self->path($dir);
+  mkdir($path) or croak "$path: $!";
+  $self->{directories}{$dir} = 1;
+}
+
+sub check_file {
+    my ($self,$file) = @_;
+    my $rv;
+    if (-f $self->path($file)) {
+      $rv = $self->{files}{$file} = 1;
+    } else {
+      $rv = $self->{files}{$file} = 0;
+    }
+    return $rv;
+}
+
+sub check_directory {
+    my ($self,$dir) = @_;
+    my $rv;
+    if (-d $self->path($dir)) {
+      $rv = $self->{directories}{$dir} = 1;
+    } else {
+      $rv = $self->{directories}{$dir} = 0;
+    }
+    return $rv;
+}
+
 sub clean {
     my $self = shift;
     foreach my $file ( keys %{$self->{files}} ) {
     	unlink $self->path($file);
     };
+    foreach my $dir ( keys %{$self->{directories}} ) {
+    	rmdir $self->path($dir);
+    };
     rmdir $self->{dir};
 }
     
+sub _path_map {
+    my $self = shift;
+    my %path;
+    while (my ($k,$v) = each %{$self->{files}}) {
+	$path{ $self->name($k) } = $v;
+    };
+    while (my ($k,$v) = each %{$self->{directories}}) {
+	$path{ $self->name($k) } = $v;
+    };
+    return \%path;
+}
+
 sub count_unknown {
     my $self = shift;
+    my $path = $self->_path_map;
     opendir my($dh), $self->{dir} or croak "$self->{dir}: $!";
 
-    my %path = map {$self->name($_)=>$self->{files}{$_}} keys %{$self->{files}};
     my $count = 0;
     while (my $file = readdir($dh)) {
 	next if $file eq '.';
 	next if $file eq '..';
-	next if $path{$file};
+	next if $path->{$file};
 	++ $count;
     }
     return $count;
@@ -115,6 +158,9 @@ sub count_missing {
     while (my($file,$has) = each %{$self->{files}}) {
 	++ $count if ($has and not(-f $self->path($file)));
     }
+    while (my($file,$has) = each %{$self->{directories}}) {
+	++ $count if ($has and not(-d $self->path($file)));
+    }
     return $count;
 }
 
@@ -130,20 +176,45 @@ sub remove_files {
   return $count;
 }
 
+sub remove_directories {
+  my $self = shift;
+  my $count = 0;
+  foreach my $file (@_) {
+    my $path = $self->path($file);
+    $self->{directories}{$file} = 0;
+    $count ++ if rmdir($path);
+  }
+  return $count;
+}
+
 ##############################
 # Test Functions
 ##############################
 
 sub has {
     my ($self,$file,$text) = @_;
+    $text = "File $file is found." unless defined $text;
     $self->builder->ok( $self->check_file($file), $text );
 }
 
 sub hasnt {
     my ($self,$file,$text) = @_;
+    $text = "File $file is not found." unless defined $text;
     $self->builder->ok( not($self->check_file($file)), $text );
 }
 
+sub has_dir {
+    my ($self,$file,$text) = @_;
+    $text = "Directory $file is found." unless defined $text;
+    $self->builder->ok( $self->check_directory($file), $text );
+}
+
+sub hasnt_dir {
+    my ($self,$file,$text) = @_;
+    $text = "Directory $file is not found." unless defined $text;
+    $self->builder->ok( not($self->check_directory($file)), $text );
+}
+
 sub clean_ok {
     my ($self,$text) = @_;
     $self->builder->ok($self->clean, $text);
@@ -153,6 +224,7 @@ sub is_ok {
     my $self = shift;
     my $name = shift;
     my $test = $self->builder;
+    $name = "Directory is consistent" unless defined $name;
 
     my @miss;
     while (my($file,$has) = each %{$self->{files}}) {
@@ -160,21 +232,28 @@ sub is_ok {
 	    push @miss, $file;
 	}
     }
+    my @miss_d;
+    while (my($file,$has) = each %{$self->{directories}}) {
+	if ($has and not(-d $self->path($file))) {
+	    push @miss_d, $file;
+	}
+    }
 
     opendir my($dh), $self->{dir} or croak "$self->{dir}: $!";
 
-    my %path = map {$self->name($_)=>$self->{files}{$_}} keys %{$self->{files}};
+    my $path = $self->_path_map;
     my @unknown;
     while (my $file = readdir($dh)) {
 	next if $file eq '.';
 	next if $file eq '..';
-	next if $path{$file};
+	next if $path->{$file};
 	push @unknown, $file;
     }
 
-    my $rv = $test->is_num(@miss+@unknown, 0, $name);
+    my $rv = $test->ok((@miss+@unknown) == 0, $name);
     unless ($rv) {
-	$test->diag("Missing: $_") foreach @miss;
+	$test->diag("Missing file: $_") foreach @miss;
+	$test->diag("Missing directory: $_") foreach @miss_d;
 	$test->diag("Unknown file: $_") foreach @unknown;
     }
     return $rv;
@@ -196,18 +275,24 @@ Test::Directory - Perl extension for maintaining test directories.
 
  my $dir = Test::Directory->new($path);
  $dir->touch($src_file);
- My::Module::something( $dir->path($src_file) );
- $dir->has_ok($src_file); #is source still there?
- $dir->has_ok($dst_file); #did my module create dst?
-
+ My::Module::something( $dir->path($src_file), $dir->path($dst_file) );
+ $dir->has_ok($dst_file);   #did my module create dst?
+ $dir->hasnt_ok($src_file); #is source still there?
 
 =head1 DESCRIPTION
 
-Sometimes, testing code involves making sure that files are created and
-deleted as expected.  This module simplifies maintaining test directories by
-tracking their status as they are modified or tested with this API, making
-it simple to test both individual files, as well as to verify that there are
-no missing or unknown files.
+Testing code can involve making sure that files are created and deleted as
+expected.  Doing this manually can be error prone, as it's easy to forget a
+file, or miss that some unexpected file was added. This module simplifies
+maintaining test directories by tracking their status as they are modified
+or tested with this API, making it simple to test both individual files, as
+well as to verify that there are no missing or unknown files.
+
+The idea is to use this API to create a temporary directory and
+populate an initial set of files.  Then, whenever something in the directory
+is changes, use the test methods to verify that the change happened as
+expected.  At any time, it is simple to verify that the contents of the
+directory are exactly as expected.
 
 Test::Directory implements an object-oriented interface for managing test
 directories.  It tracks which files it knows about (by creating or testing
@@ -232,8 +317,10 @@ scope; see the I<clean> method below for details.
 Create a new instance pointing to the specified I<$path>. I<$options> is 
 an optional hashref of options.
 
-I<$path> will be created it necessary.  If I<$options>->{unique} is set, it is
-an error for I<$path> to already exist.
+I<$path> will be created (or the constructor will die).  By default,
+I<$options>->{unique} is true, so it is an error for I<$path> to already
+exist; setting this option to false will allow reusing an existing
+directory.
 
 =back
 
@@ -253,35 +340,63 @@ supports the following:
 
 =over 8
 
-=item B<time> I<$timestamp>
+=item B<time> => I<$timestamp>
 
-Passed to L<utime> to set the files access and modification times.
+Passed to L<perlfunc/utime> to set the files access and modification times.
 
-=item B<content> I<$data>
+=item B<content> => I<$data>
 
 Write I<$data> to the file.
 
 =back
 
-=item B<name>(I<FILE>)
+=item B<mkdir>(I<$directory>)
 
-Returns the name of the I<FILE>, relative to the directory; including any
-template substitutions.  I<FILE> need not exist.
+Create the specified I<$directory>; dies if I<mkdir> fails.
 
-=item B<path>(I<FILE>)
+=item B<name>(I<$file>)
 
-Returns the path for the I<FILE>, including the directory name and any template
-substitutions.  I<FILE> need not exist.
+Returns the name of the I<$file>, relative to the directory; including any
+template substitutions.  I<$file> need not exist.  This method is used
+internally by most other methods to translate file paths.
+
+For portability, this method implicitly splits the path on UNIX-style /
+seperators, and rejoins it with the local directory seperator.
+
+Absent any template or seperator substitution, the returned value would be
+equivalent to I<$file>.
+
+=item B<path>(I<$file>)
+
+Returns the path for the I<$file>, including the directory name and any
+substitutions.  I<$file> need not exist.
 
 =item B<check_file>(I<$file>)
 
 Checks whether the specified I<$file> exists, and updates its state
 accordingly.  Returns true if I<$file> exists, false otherwise.
 
+This method is used internally by the corresponding test methods.
+
+=item B<check_directory>(I<$directory>)
+
+Checks whether the specified I<$directory> exists, and updates its state
+accordingly.  Returns true if I<$directory> exists, false otherwise.
+
+This method is used internally by the corresponding test methods.
+
+Note that replacing a file with a directory, or vice versa, would require
+calling both I<check_file> and I<check_directory> to update the state to
+reflect both changes.
+
 =item B<remove_files>(I<$file>...) 
 
 Remove the specified $I<file>s; return the number of files removed.
 
+=item B<remove_directories>(I<$directory>...) 
+
+Remove the specified $I<directories>s; return the number of directories removed.
+
 =item B<clean>
 
 Remove all known files, then call I<rmdir> on the directory; returns the
@@ -294,7 +409,9 @@ This method is called automatically when the object goes out of scope.
 
 =item B<count_missing>
 
-Returns a count of the unknown or missing files.
+Returns a count of the unknown or missing files and directories.  Note that
+files and directores are interchangeable when counting missing files, but
+not when counting unknown files.
 
 =back
 
@@ -310,15 +427,24 @@ L<Test::Builder>'s I<ok> and I<diag> methods to generate output.
 =item B<hasnt>(I<$file>, I<$test_name>)
 
 Verify the status of I<$file>, and update its state.  The test will pass if
-the state is expected.
+the state is expected.  If I<$test_name> is undefined, a default will be
+generated.
+
+=item B<has_dir>  (I<$directory>, I<$test_name>);
+
+=item B<hasnt_dir>(I<$directory>, I<$test_name>);
+
+Verify the status of I<$directory>, and update its state.  The test will
+pass if the state is expected.  If I<$test_name> is undefined, a default will be
+generated.
 
 =item B<is_ok>(I<$test_name>)
 
 Pass if the test directory has no missing or extra files.
 
-=item B<clean_ok>([I<TEXT>])
+=item B<clean_ok>([I<$test_name>])
 
-Equivalent to ok(clean,I<TEXT>)
+Equivalent to ok(clean,I<$test_name>)
 
 =back
 
